# -*- coding: utf-8 -*-
"""
Exceptions.
"""


class Dump2PolarionException(Exception):
    """dump2polarion exception."""


class NothingToDoException(Dump2PolarionException):
    """nothing to do exception."""
